<?php

/**
 * @file
 * Administrative page callbacks for the Monolog Database module.
 */

/**
 * Page callback; Displays the log messages.
 */
function monolog_database_view() {
  $build = array();
  $rows = array();
  try {

    composer_manager_register_autoloader();
    module_load_include('inc', 'monolog', 'monolog.admin');
    $levels = monolog_level_options();

    $header = array(
      '',
      array('data' => t('Channel'), 'field' => 'log.channel'),
      array('data' => t('Level'), 'field' => 'log.level'),
      array('data' => t('Date'), 'field' => 'log.mid', 'sort' => 'desc'),
      t('Message'),
      array('data' => t('User'), 'field' => 'u.name'),
      array('data' => t('Operations')),
    );

    $query = db_select('monolog_log', 'log')->extend('PagerDefault')->extend('TableSort');
    $query->leftJoin('users', 'u', 'log.ctx_uid = u.uid');
    $query
      ->fields('log', array('mid', 'channel', 'timestamp', 'message', 'level', 'ctx_link', 'ctx_uid'))
      ->addField('u', 'name');

    $result = $query
      ->limit(50)
      ->orderByHeader($header)
      ->execute();

    foreach ($result as $message) {

      $account = new stdClass();
      $account->uid = $message->ctx_uid;
      $account->name = $message->name;

      $rows[] = array(
        'data' => array(
          array('class' => 'icon'),
          check_plain($message->channel),
          isset($levels[$message->level]) ? $levels[$message->level] : '',
          format_date($message->timestamp, 'short'),
          theme('monolog_database_message', array('message' => $message->message)),
          theme('username', array('account' => $account)),
          filter_xss($message->ctx_link),
        ),
        //'class' => array(drupal_html_class('dblog-' . $dblog->type), $classes[$dblog->severity]),
      );
    }
  }
  catch (Exception $e) {
    drupal_set_message($e->getMessage(), 'error');
  }

  $build['log_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'monolog-database-view'),
    '#empty' => t('No log messages available.'),
  );

  $build['log_pager'] = array('#theme' => 'pager');

  return $build;
}

/**
 * Returns HTML for a log message.
 *
 * @param array $variables
 *   An associative array containing:
 *   - message: The message that was logged.
 *
 * @ingroup themeable
 */
function theme_monolog_database_message($variables) {
  return truncate_utf8(filter_xss($variables['message'], array()), 72, TRUE, TRUE);
}
