<?php

/**
 * Monolog extension for use with Drupal.
 */

namespace Drupal\Monolog\Handler;

use Monolog\Logger;
use Monolog\Formatter\NormalizerFormatter;
use Monolog\Handler\AbstractProcessingHandler;

/**
 * Handler that logs records to a Drupal database.
 */
class DrupalDatabaseHandler extends AbstractProcessingHandler
{
    /**
     * The monolog table's schema definition.
     *
     * @var array
     */
    protected $_schema;

    /**
     * {@inheritDoc}
     */
    public function __construct($level = Logger::DEBUG, $bubble = true)
    {
        $this->_schema = drupal_get_schema('monolog_log');
        parent::__construct($level, $bubble);
    }

    /**
     * Returns the monolog table's schema definition.
     *
     * @return array
     */
    public function getSchema()
    {
        return $this->_schema;
    }

    /**
     * {@inheritDoc}
     */
    protected function write(array $record)
    {
        $formatted = &$record['formatted'];
        $context = &$formatted['context'];

        $fields = array(
            'channel' => $formatted['channel'],
            'timestamp' => $record['datetime']->getTimestamp(),
            'message' => $formatted['message'],
            'level' => $formatted['level'],
            'extra' => serialize($formatted['extra']),
            'extra_ctx' => array(),
            'ctx_type' => '',
            'ctx_uid' => 0,
            'ctx_request_uri' => '',
            'ctx_referer' => '',
            'ctx_ip' => '',
            'ctx_link' => '',
            'ctx_request_id' => '',
        );

        foreach ($context as $key => $value) {
            $ctx_field = 'ctx_' . $key;
            if (isset($this->_schema['fields'][$ctx_field])) {
              $fields[$ctx_field] = $value;
            } else {
              $fields['extra_ctx'][$ctx_field] = $value;
            }
        }

        $fields['extra_ctx'] = serialize($fields['extra_ctx']);

        \Database::getConnection('default', 'default')
            ->insert('monolog_log')
            ->fields($fields)
            ->execute();
    }

    /**
     * {@inheritDoc}
     */
    protected function getDefaultFormatter()
    {
        return new NormalizerFormatter();
    }
}
